// =====================================================
//
//   miniLA_win - Frequency Measurement
//
//   (c) miniLA Team
//
// =====================================================
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this package; see the file COPYING.  If not, write to
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.

unit dlgDecFreq;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons,
  uDecUtils, Spin;

type
  TfrmDecFreq = class(TForm)
    cbChannel: TComboBox;
    Label1: TLabel;
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    GroupBox1: TGroupBox;
    gbDecodeRange: TGroupBox;
    rbAllData: TRadioButton;
    rbCursors: TRadioButton;
    procedure btnOKClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    ChannelSel:  TChannelSel;

    decoded_channel 	: integer;
    channel		: cardinal;
    channel_last	: cardinal;

    function D_i(val:byte):cardinal;
    function D_o(val:cardinal):byte;

    function loc_data_find_change:boolean;
  public
    { Public declarations }
  end;

var
  frmDecFreq: TfrmDecFreq;

implementation

uses
   uUtils,
   uLATypes,
   uDecData,
   uINIFile,
   dlgDecDisplay,
   dlgMain, dlgWave;

{$R *.dfm}

procedure TfrmDecFreq.FormCreate(Sender: TObject);
begin
   cbChannel.ItemIndex  := INIFile.ReadInteger('DEC_FREQ','CHANNEL', 0);
   rbAllData.Checked    := INIFile.ReadBool('DEC_FREQ','DEC_FULL_RANGE', true);
   rbCursors.Checked	:= not(rbAllData.Checked);
end;


procedure TfrmDecFreq.FormDestroy(Sender: TObject);
begin
   INIFile.WriteInteger('DEC_FREQ','CHANNEL', cbChannel.ItemIndex);
   INIFile.WriteBool('DEC_FREQ','DEC_FULL_RANGE', rbAllData.Checked);
end;

procedure TfrmDecFreq.FormActivate(Sender: TObject);
begin
   ChannelSel := TChannelSel.Create;
   ChannelSel.Add(cbChannel);
end;

function TfrmDecFreq.D_i(val:byte):cardinal;
begin
   result := val shl decoded_channel;
end;

function TfrmDecFreq.D_o(val:cardinal):byte;
begin
   result := (val shr decoded_channel) and $1;
end;

function TfrmDecFreq.loc_data_find_change:boolean;
var
   read_err	: boolean;
begin
   channel_last := channel;
   read_err := DecDataFindChange(D_i(1));
   channel := D_o(sample);
   result := read_err;
end;


procedure TfrmDecFreq.btnOKClick(Sender: TObject);
var
   fs           : double;
   edge_count   : cardinal;
   period_count : cardinal;
   low_width    : cardinal;
   high_width   : cardinal;
   low_samples  : cardinal;
   high_samples : cardinal;
   prev_edge_n  : cardinal;
   first_edge   : cardinal;
   last_edge    : cardinal;

   err_msg	: string;

begin
   err_msg := '';
   frmDecDisplay.lbResults.Clear;

   decoded_channel := ChannelSel.GetChannelNumber(cbChannel);
   if (decoded_channel < 0) then
      err_msg := 'channel';

   if err_msg <> '' then
    begin
      MessageDlg('Invalid '+err_msg, mtError,[mbOK],0);
      exit;
    end;

   ChannelSel.Free;
   Hide;
   frmMain.StatusBar1.Panels[5].Text := 'Measuring Frequency ...';
   frmMain.StatusBar1.Repaint;

   if rbAllData.Checked then
      DecDataInit(0, frmMain.DataHold.MaxPosition)	// initialize sample read routine
   else
      DecDataInit(frmWave.LogPanel.Cursors[0].Position,
		  frmWave.LogPanel.Cursors[1].Position);

   frmDecDisplay.lbResults.Items.Add(Format('%d (%ss) start of measurment', [sample_n, TmToStr(time_p,2)]));

   edge_count := 0;
   low_samples := 0;
   high_samples := 0;

   // get initial value
   channel := D_o(sample);

   while (true) do
   begin
     if loc_data_find_change() then
       break;

     inc(edge_count);

     if edge_count = 1 then
       first_edge := sample_n
     else
     begin
       if D_o(sample) > 0 then
         low_width := sample_n - prev_edge_n
       else
         high_width := sample_n - prev_edge_n;

       if edge_count mod 2 = 1 then
       begin
         last_edge := sample_n;
         low_samples := low_samples + low_width;
         high_samples := high_samples + high_width;
       end;
     end;

     prev_edge_n := sample_n;
   end;

   if edge_count < 3 then
   begin
     frmDecDisplay.lbResults.Items.Add(Format('%d (%ss) end of measurement', [sample_n, TmToStr(time_p,2)]));
     frmDecDisplay.lbResults.Items.Add('No full period of signal found.')
   end;

   if edge_count >= 3 then
   begin
     fs := frmWave.LogPanel.Timebase;
     period_count := (edge_count - 1) div 2;
     frmDecDisplay.lbResults.Items.Add(Format('%d (%ss) first edge', [first_edge, TmToStr(first_edge / fs, 2)]));
     frmDecDisplay.lbResults.Items.Add(Format('%d (%ss) last edge', [last_edge, TmToStr(last_edge / fs, 2)]));
     frmDecDisplay.lbResults.Items.Add(Format('%d (%ss) end of measurement', [sample_n, TmToStr(time_p, 2)]));
     frmDecDisplay.lbResults.Items.Add(Format('Frequency: %sHz', [TmToStr(period_count / (last_edge - first_edge) * fs, 6)]));
     frmDecDisplay.lbResults.Items.Add(Format('Period: %ss', [TmToStr((last_edge - first_edge) / period_count / fs, 6)]));
     frmDecDisplay.lbResults.Items.Add(Format('Freq Resolution: %sHz', [TmToStr(fs * period_count / (high_samples + low_samples) / (high_samples + low_samples), 1)]));
     frmDecDisplay.lbResults.Items.Add(Format('Time Resolution: %ss', [TmToStr(1 / fs / period_count, 1)]));
     frmDecDisplay.lbResults.Items.Add(Format('DC: %.3f%%', [100 * high_samples / (low_samples + high_samples)]));
     frmDecDisplay.lbResults.Items.Add(Format('Avg high width: %ss', [TmToStr(high_samples / period_count / fs, 6)]));
     frmDecDisplay.lbResults.Items.Add(Format('Avg low width: %ss', [TmToStr(low_samples / period_count / fs, 6)]));
     frmDecDisplay.lbResults.Items.Add(Format('Periods: %u', [period_count]));
   end;

   ModalResult := mrOK;
   frmDecDisplay.Caption := 'Frequency Measurement';
   frmDecDisplay.Show;
   frmMain.StatusBar1.Panels[5].Text := '';
   frmMain.StatusBar1.Repaint;
end;


procedure TfrmDecFreq.btnCancelClick(Sender: TObject);
begin
   ChannelSel.Free;
end;





end.

