// =====================================================
//
//   miniLA_win - Text display unit
//
//   (c) miniLA Team
//
// =====================================================
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this package; see the file COPYING.  If not, write to
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.

// TODO:
//
// Tie selected line with particular position so it stays enabled
// on the particular line even when scrolling to non-visible lines


unit dlgListView;

interface

uses
   Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
   Dialogs, StdCtrls, ComCtrls, ExtCtrls, Math,
   uDisplay,
   dlgChannel,
   uLATypes,
   uUtils, WOwnrScrllBar, Menus;

type
  TfrmList = class(TForm)
    sbrList: TWOwnrScrllBar;
    lbList: TListBox;
    ListPopupMenu: TPopupMenu;
    PlaceCursor11: TMenuItem;
    PlaceCursor21: TMenuItem;
    ListMenu: TMainMenu;
    N1: TMenuItem;
    ViewCursor11: TMenuItem;
    ViewCursor21: TMenuItem;
    ViewTriggerCursor1: TMenuItem;
    mnStateView: TMenuItem;
    N2: TMenuItem;
    Panel1: TPanel;
    Splitter1: TSplitter;
    lbSigNames: TListBox;
    mnRelativeTimes: TMenuItem;
    procedure sbrListChange(Sender: TObject);
    procedure lbListKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure WMGetDlgCode(var Msg: TWMGetDlgCode);
    procedure lbListDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure PlaceCursor11Click(Sender: TObject);
    procedure ViewCursor11Click(Sender: TObject);
    procedure mnStateViewClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure lbListDblClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure Splitter1Moved(Sender: TObject);
    procedure lbSigNamesDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure mnRelativeTimesClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    FSigVisibleLines : word;
    FSigPos: array[0..31] of word;              // position of signal name/value
    FSigWidth: array[0..31] of word;            // width of signal names
    FMask : cardinal;

    Fmax_visible_lines : word;
    FStateView : boolean;
    FRelativeTimes : boolean;
    FCursors : array[0..2] of int64;            // scrollbar cursors

    procedure PrintData;
    function AChannelValue(pos: int64; ch: byte): string;
    procedure PrintSigNames;
    procedure UpdateScrPosition(pos: int64);
    function PosToChg(pos: int64): int64;
    function ChgToPos(chg: int64): int64;
    function SelectedLine: integer;
    procedure MouseWheel(var Msg: TWMMouseWheel); message CM_MouseWheel;


  public
    { Public declarations }
    procedure Initialize;
    procedure SetCursorPosition(No   : byte; Pos:int64);

  end;

var
  frmList: TfrmList;

implementation

uses dlgMain, dlgWave, uINIFile;

{$R *.dfm}

//=================================================================
procedure TfrmList.FormCreate(Sender: TObject);
//=================================================================
begin
   FStateView := INIFile.ReadBool('GUI', 'LV_STATEVIEW', true);
   FRelativeTimes := INIFile.ReadBool('GUI', 'LV_RELTIMES', false);

   mnStateView.Checked := FStateView;

   if FStateView then
    begin
      mnRelativeTimes.Enabled := true;
      mnRelativeTimes.Checked := FRelativeTimes;
    end
   else
    begin
      mnRelativeTimes.Enabled := false;
      mnRelativeTimes.Checked := false;
    end;

end;


//=================================================================
procedure TfrmList.FormDestroy(Sender: TObject);
//=================================================================
begin
   INIFile.WriteBool('GUI', 'LV_STATEVIEW', FStateView);
   INIFile.WriteBool('GUI', 'LV_RELTIMES', FRelativeTimes);
end;

//=================================================================
// Convert position to number of changes
//=================================================================
function TfrmList.PosToChg(pos: int64): int64;
var
   change_num   : int64;
   change_found : boolean;
begin
   change_num := 0;
   frmMain.DataHold.GetSampleValue(0);                  // move position to start

   repeat
     inc(change_num);
     change_found := frmMain.DataHold.FindChange(FMask, frmMain.DataHold.MaxPosition, true);
   until not(change_found and (pos > frmMain.DataHold.Position));

   if (change_num > 0) and (frmMain.DataHold.Position > pos) then
      dec(change_num);

   result := change_num;

end;


//=================================================================
// Convert number of changes to position
//=================================================================
function TfrmList.ChgToPos(chg: int64): int64;
var
   change_num   : int64;
   change_found : boolean;
begin
   change_num := 0;
   frmMain.DataHold.GetSampleValue(0);                  // move position to start

   if chg>0 then
      repeat
         inc(change_num);
         change_found := frmMain.DataHold.FindChange(FMask, frmMain.DataHold.MaxPosition, true);
      until not (change_found and (change_num<chg));

   result := frmMain.DataHold.Position;

end;

//=================================================================
// Return number of selected line
//=================================================================
function TfrmList.SelectedLine: integer;
var
   i : word;
begin
   i := 0;
   while (i<lbList.Count) and not(lbList.Selected[i]) do
      inc(i);

   if i<lbList.Count then
      result := i
   else
      result := -1;
end;



//=================================================================
procedure TfrmList.FormResize(Sender: TObject);
//=================================================================
begin
   Initialize;
end;


//=================================================================
procedure TfrmList.Initialize;
//=================================================================
var
   FMaxVal : int64;
   change_found : boolean;

   mwidth : word;
   tpos : word;
   i,ns : byte;
begin
   lbList.Clear;

   // find width of signals and calculate their position
   // create search mask
   tpos := 27;                                                          // 1st signal name

   FMask := 0;
   for i:= 0 to frmWave.LogPanel.Display.Dataset.NumberOfLines-1 do
    begin
      FMask := FMask or frmWave.Logpanel.Display.Dataset.Data[i].Mask;
      if frmWave.Logpanel.Display.Dataset.Data[i].Group then
       begin
         ns := frmWave.Logpanel.Display.Dataset.Data[i].NumberOfSignals;
         case frmWave.Logpanel.Display.Dataset.Data[i].Base of
            0: // bin
               mwidth := ns + 1;
            1: // oct
               mwidth := ((ns+2) div 3) + 1;
            2: //dec
               mwidth := round(log10(IntPower(2, ns) - 1) + 0.5) + 1;
            3: //hex
               mwidth := ((ns+3) div 4) + 1;
         end;
       end
      else
         mwidth := 1;

      FSigWidth[i] := mwidth;
      FSigPos[i] := tpos;
      tpos := tpos + mwidth + 3;
   end;


   if not(FStateView) then
      FMaxVal := frmMain.DataHold.MaxPosition
   else
    begin
      // count number of change events
      FMaxVal := 0;
      frmMain.DataHold.GetSampleValue(0);  // move position to start

      repeat
        inc(FMaxVal);
        change_found := frmMain.DataHold.FindChange(FMask, frmMain.DataHold.MaxPosition, true);
      until not(change_found);
    end;



   Fmax_visible_lines := lbList.Height div lbList.ItemHeight;

   // set scrollbar limits
   sbrList.Min:=0;
   sbrList.PageSize:=0;				                        // prevent exceptions when setting max.value
   sbrList.Max:=FMaxVal;

   sbrList.SmallChange := 1;
   sbrList.LargeChange := (8*Fmax_visible_lines) div 10; 	        // 80% visible lines
   sbrList.PageSize := Fmax_visible_lines - 1;			        // 100% of visible lines - 1
   sbrList.Enabled := (sbrList.Max > Fmax_visible_lines);
   sbrList.Repaint;     // note: change of enable doesn't repaint scrollbar


   // Signal Names
   FSigVisibleLines := lbSigNames.Height div lbSigNames.ItemHeight;
   if (FSigVisibleLines > frmWave.LogPanel.Display.Dataset.NumberOfLines) then
      FSigVisibleLines := frmWave.LogPanel.Display.Dataset.NumberOfLines;


   PrintSigNames;
   PrintData;
end;


//=================================================================
function TfrmList.AChannelValue(pos: int64; ch: byte): string;
//=================================================================
var
   str: string;
   i: byte;
begin
   str := ChannelValueToStr(pos, ch);
   while length(str) < FSigWidth[ch] do
      insert(' ', str, 1);

   result := str;

end;

//=================================================================
procedure TfrmList.PrintSigNames;
//=================================================================
var
   i, j: integer;
   line: string;
   chstr: string;

begin
   lbSigNames.Clear;
   for i:= 0 to FSigVisibleLines-1 do
    begin
      j := i;
      line := '';
      while (j < frmWave.LogPanel.Display.Dataset.NumberOfLines) do
       begin
         chstr := '|' + frmWave.LogPanel.Display.Dataset.Data[j].Name;

         // adjust name length if necessary
         if ((j + FSigVisibleLines) < frmWave.LogPanel.Display.Dataset.NumberOfLines) then      // not last signal on line
            if length(chstr) > (FSigPos[j + FSigVisibleLines] - FSigPos[j] - 1) then
               SetLength(chstr, FSigPos[j + FSigVisibleLines] - FSigPos[j] - 1);

         // move to position
         while length(line) < FSigPos[j] do
            line := line + ' ';

         line := line + chstr;
         j := j + FSigVisibleLines;
       end;
      lbSigNames.Items.Add(line);
    end;
end;

//=================================================================
procedure TfrmList.lbSigNamesDrawItem(Control: TWinControl; Index: Integer;
//=================================================================
  Rect: TRect; State: TOwnerDrawState);
begin
  with (Control as TListBox) do
   begin
    Canvas.Font.Color := clWindowText;
    Canvas.Brush.Color := clWindow;
    Canvas.FillRect(Rect);
    Canvas.TextOut(Rect.Left,Rect.Top,Items[Index]);
   end;

end;

//=================================================================
procedure TfrmList.PrintData;
//=================================================================
var
   smp          : cardinal;
   line         : string;
   i            : integer;
   change_found : boolean;

   lpos,pos     : int64;
   shown_lines  : word;

begin
   // display lines
   if not(FStateView) then
      pos := sbrList.Position
   else
    begin
      if sbrList.Position > 0 then
       begin
         lpos := ChgToPos(sbrList.Position-1);
         frmMain.DataHold.FindChange(FMask, frmMain.DataHold.MaxPosition, true);        // seek to next change
         pos := frmMain.DataHold.Position;
       end
      else
       begin
         lpos := 0;
         pos := ChgToPos(0);
       end;
    end;

   shown_lines := 0;
   lbList.Clear;
   change_found := true;
   repeat
      if not(FRelativeTimes) or not(FStateView) then
         Line := Format('%10d (%10ss)  ', [pos-frmWave.LogPanel.Cursors[2].Position,
                                           TmToStr((pos-frmWave.LogPanel.Cursors[2].Position)/frmWave.LogPanel.TimeBase,3)])
      else
         Line := Format('+%10d (+%10ss) ', [pos-lpos,
                                           TmToStr((pos-lpos)/frmWave.LogPanel.TimeBase,3)]);

      smp := frmMain.DataHold.GetSampleValue(pos);

      for i:=0 to frmWave.LogPanel.Display.Dataset.NumberOfLines-1 do
         Line := Line + AChannelValue(smp, i) + '   ';

      lbList.Items.Add(Line);

      // seek to next change
      inc(shown_lines);
      if not(FStateView) then
         inc(pos)
      else
       begin
         change_found := frmMain.DataHold.FindChange(FMask, frmMain.DataHold.MaxPosition, true);
         lpos := pos;
         pos := frmMain.DataHold.Position;
       end;

   until not((pos<frmMain.DataHold.MaxPosition) and (shown_lines < Fmax_visible_lines) and change_found);
   lbList.Update;

end;

//=================================================================
procedure TfrmList.lbListDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
//=================================================================

begin
  with (Control as TListBox) do
  begin
    if not(FStateView) then
     begin
       if (sbrList.Position+Index) = frmWave.LogPanel.Cursors[2].Position then
          Canvas.Font.Color := frmWave.LogPanel.Cursors[2].ColorC
       else if (sbrList.Position+Index) = frmWave.LogPanel.Cursors[0].Position then
          Canvas.Font.Color := frmWave.LogPanel.Cursors[0].ColorC
       else if (sbrList.Position+Index) = frmWave.LogPanel.Cursors[1].Position then
          Canvas.Font.Color := frmWave.LogPanel.Cursors[1].ColorC
       else if odSelected in State then
          Canvas.Font.Color := clHighlightText
       else
          Canvas.Font.Color := clWindowText;
     end
    else
     begin
       if (sbrList.Position+Index) = Fcursors[2] then
          Canvas.Font.Color := frmWave.LogPanel.Cursors[2].ColorC
       else if (sbrList.Position+Index) = Fcursors[0] then
          Canvas.Font.Color := frmWave.LogPanel.Cursors[0].ColorC
       else if (sbrList.Position+Index) = Fcursors[1] then
          Canvas.Font.Color := frmWave.LogPanel.Cursors[1].ColorC
       else if odSelected in State then
          Canvas.Font.Color := clHighlightText
       else
          Canvas.Font.Color := clWindowText;
     end;

    Canvas.FillRect(Rect);
    Canvas.TextOut(Rect.Left,Rect.Top,Items[Index]);
  end;
end;


//=================================================================
procedure TfrmList.sbrListChange(Sender: TObject);
//=================================================================
begin
   PrintData;
end;


//=================================================================
// update scrollbar position without firing onchange event
//=================================================================
procedure TfrmList.UpdateScrPosition(pos: int64);
var
   oldNotify	:TNotifyEvent;
begin
   oldNotify := frmList.sbrList.OnChange;
   frmList.sbrList.OnChange := nil;
   frmList.sbrList.Position:=pos;
   frmList.sbrList.OnChange := oldNotify;
end;



//=================================================================
procedure TfrmList.SetCursorPosition(No :byte; Pos:int64);
//=================================================================
var
   cpos, lpos : int64;
begin
   if not(FStateView) then
      cpos := pos
   else
      cpos := PosToChg(pos);

   sbrList.SetCursorPosition(No, cpos);
   lpos := FCursors[No];
   FCursors[No] := cpos;

   // repaint if new or old cursor position visible 
   if    ((sbrList.Position <= cpos) and (cpos < (sbrList.Position + Fmax_visible_lines)))
      or ((sbrList.Position <= lpos) and (lpos < (sbrList.Position + Fmax_visible_lines))) then
      lbList.Repaint;

end;



//=================================================================
// Mouse wheel
//=================================================================
procedure TfrmList.MouseWheel(var Msg: TWMMouseWheel);
var
 Delta: integer;
 IsUp: boolean;
 sel : integer;
begin
   //Set the wheel value
   Delta := Msg.WheelDelta div WHEEL_DELTA;
   IsUp := Delta > 0;
   if Delta < 0 then Delta := Delta * -1;

//   sel := SelectedLine;

   if IsUp then
    begin
      if (sbrList.Position > 0) then
       begin
         UpdateScrPosition(sbrList.Position - 1);
         PrintData;
         //lbList.Selected[0] := true;
       end;
    end
   else
    begin
      if (lbList.Count = Fmax_visible_lines) and (sbrList.Position < sbrList.Max) then
       begin
         UpdateScrPosition(sbrList.Position + 1);
         PrintData;
         //lbList.Selected[lbList.Count-1] := true;
       end;
    end;

   Msg.Result := 1;
end;


//=================================================================
procedure TfrmList.lbListDblClick(Sender: TObject);
//=================================================================
var
   sel, nsel : int64;
begin
   sel := SelectedLine;
   if sel<0 then
      sel := 0;

   sel := sel + sbrList.Position;

   if not(FStateView) then
      nsel := sel
   else
      nsel := ChgToPos(sel);

   frmWave.LogPanel.SetCenterPosition(nsel);
   frmWave.LogPanel.SetCursorPosition(3, nsel);
end;


//=================================================================
// enable reception of cursor keys keydown events
//=================================================================
procedure TfrmList.WMGetDlgCode(var Msg: TWMGetDlgCode);
begin
   msg.result:= DLGC_WANTARROWS;
end;


//=================================================================
procedure TfrmList.lbListKeyDown(Sender: TObject; var Key: Word;
//=================================================================
  Shift: TShiftState);
begin
   if key in [VK_HOME, VK_UP, VK_PRIOR, VK_DOWN, VK_NEXT, VK_END] then
    begin
      case Key of
        VK_UP:
          begin
             if lbList.Selected[0] and (sbrList.Position > 0) then
              begin
                UpdateScrPosition(sbrList.Position - 1);
                PrintData;
                lbList.Selected[0] := true;
                key := 0;
              end;
          end;

        VK_PRIOR:       // PgUp
          begin
             if lbList.Selected[0] and (sbrList.Position > 0) then
              begin
                if (sbrList.Position > Fmax_visible_lines) then
                   UpdateScrPosition(sbrList.Position - Fmax_visible_lines)
                else
                   UpdateScrPosition(0);
                PrintData;
                lbList.Selected[0] := true;
                key := 0;
              end;
          end;

        VK_HOME:
          begin
            UpdateScrPosition(0);
            PrintData;
            lbList.Selected[0] := true;
            key := 0;
          end;

        VK_DOWN:
          begin
             if lbList.Count = Fmax_visible_lines then
                if lbList.Selected[Fmax_visible_lines-1] and (sbrList.Position < sbrList.Max) then
                 begin
                   UpdateScrPosition(sbrList.Position + 1);
                   PrintData;
                   lbList.Selected[lbList.Count-1] := true;
                   key := 0;
                 end;
          end;

        VK_NEXT:       // PgDown
          begin
             if lbList.Count = Fmax_visible_lines then
                if lbList.Selected[Fmax_visible_lines-1] and (sbrList.Position < sbrList.Max) then
                 begin
                   if (sbrList.Position < sbrList.Max-Fmax_visible_lines) then
                      UpdateScrPosition(sbrList.Position + Fmax_visible_lines)
                   else
                      UpdateScrPosition(sbrList.Max-Fmax_visible_lines);
                   PrintData;
                   lbList.Selected[lbList.Count-1] := true;
                   key := 0;
                 end;
          end;

        VK_END:
          begin
             if sbrList.Max > Fmax_visible_lines then
                UpdateScrPosition(sbrList.Max-Fmax_visible_lines)
             else
                UpdateScrPosition(sbrList.Max);

             PrintData;
             lbList.Selected[lbList.Count-1] := true;
             key := 0;
          end;
      end;

    end;
end;



//=================================================================
// Place cursor 1 or 2
//=================================================================
procedure TfrmList.PlaceCursor11Click(Sender: TObject);
var
   i : word;
   sel : int64;
begin
   i := 0;
   while (i<lbList.Count) and not(lbList.Selected[i]) do
      inc(i);

   if i<lbList.Count then
      sel := i
   else
      sel := 0;

   sel := sel + sbrList.Position;

   if not(FStateView) then
      frmWave.LogPanel.SetCursorPosition((Sender as TMenuItem).Tag, sel)
   else
      frmWave.LogPanel.SetCursorPosition((Sender as TMenuItem).Tag, ChgToPos(sel));
end;


//=================================================================
procedure TfrmList.ViewCursor11Click(Sender: TObject);
//=================================================================
var
   cpos, npos : int64;
begin
   cpos := frmWave.LogPanel.Cursors[(Sender as TMenuItem).Tag].Position;

   if FStateView then
      cpos := PosToChg(cpos);

   npos := cpos - (Fmax_visible_lines div 3);

   if npos < 0 then
      npos := 0;

   UpdateScrPosition(npos);
   PrintData;
end;


//=================================================================
procedure TfrmList.mnStateViewClick(Sender: TObject);
//=================================================================
var i : byte;
begin
   FStateView := not(FStateView);
   (Sender as TMenuItem).Checked := FStateView;

   if FStateView then
    begin
      mnRelativeTimes.Enabled := true;
      mnRelativeTimes.Checked := FRelativeTimes;
    end
   else
    begin
      mnRelativeTimes.Enabled := false;
      mnRelativeTimes.Checked := false;
    end;

   Initialize;
   for i:=0 to 2 do
      SetCursorPosition(i, frmWave.LogPanel.Cursors[i].Position);
end;


//=================================================================
procedure TfrmList.mnRelativeTimesClick(Sender: TObject);
//=================================================================
begin
   FRelativeTimes := not(FRelativeTimes);
   (Sender as TMenuItem).Checked := FRelativeTimes;
   PrintData;
end;


//=================================================================
procedure TfrmList.Splitter1Moved(Sender: TObject);
//=================================================================
begin
   Initialize;
end;



end.
